@@ -20,6 +20,7 @@ package kafka.server
 import java.nio.charset.StandardCharsets
 import java.util
 import java.util.Properties
+
 import kafka.network.SocketServer
 import kafka.security.authorizer.AclAuthorizer
 import org.apache.kafka.clients.admin.ScramMechanism
@@ -30,8 +31,9 @@ import org.apache.kafka.common.protocol.{ApiKeys, Errors}
 import org.apache.kafka.common.requests.{AlterUserScramCredentialsRequest, AlterUserScramCredentialsResponse, DescribeUserScramCredentialsRequest, DescribeUserScramCredentialsResponse}
 import org.apache.kafka.common.security.auth.{AuthenticationContext, KafkaPrincipal, KafkaPrincipalBuilder}
 import org.apache.kafka.server.authorizer.{Action, AuthorizableRequestContext, AuthorizationResult}
-import org.junit.jupiter.api.Assertions._
-import org.junit.jupiter.api.Test
+import org.junit.Assert._
+import org.junit.rules.TestName
+import org.junit.{Rule, Test}
 
 import scala.jdk.CollectionConverters._
 
@@ -44,9 +46,17 @@ class AlterUserScramCredentialsRequestTest extends BaseRequestTest {
   override def brokerPropertyOverrides(properties: Properties): Unit = {
     properties.put(KafkaConfig.ControlledShutdownEnableProp, "false")
     properties.put(KafkaConfig.AuthorizerClassNameProp, classOf[AlterCredentialsTest.TestAuthorizer].getName)
-    properties.put(KafkaConfig.PrincipalBuilderClassProp, classOf[AlterCredentialsTest.TestPrincipalBuilderReturningAuthorized].getName)
+    properties.put(KafkaConfig.PrincipalBuilderClassProp,
+      if (testName.getMethodName.endsWith("NotAuthorized")) {
+        classOf[AlterCredentialsTest.TestPrincipalBuilderReturningUnauthorized].getName
+      } else {
+        classOf[AlterCredentialsTest.TestPrincipalBuilderReturningAuthorized].getName
+      })
   }
 
+  private val _testName = new TestName
+  @Rule def testName = _testName
+
   private val saltedPasswordBytes = "saltedPassword".getBytes(StandardCharsets.UTF_8)
   private val saltBytes = "salt".getBytes(StandardCharsets.UTF_8)
   private val user1 = "user1"
@@ -65,6 +75,32 @@ class AlterUserScramCredentialsRequestTest extends BaseRequestTest {
     assertEquals(0, results.size)
   }
 
+  @Test
+  def testAlterNothingNotAuthorized(): Unit = {
+    val request = new AlterUserScramCredentialsRequest.Builder(
+      new AlterUserScramCredentialsRequestData()
+        .setDeletions(new util.ArrayList[AlterUserScramCredentialsRequestData.ScramCredentialDeletion])
+        .setUpsertions(new util.ArrayList[AlterUserScramCredentialsRequestData.ScramCredentialUpsertion])).build()
+    val response = sendAlterUserScramCredentialsRequest(request)
+
+    val results = response.data.results
+    assertEquals(0, results.size)
+  }
+
+  @Test
+  def testAlterSomethingNotAuthorized(): Unit = {
+
+    val request = new AlterUserScramCredentialsRequest.Builder(
+      new AlterUserScramCredentialsRequestData()
+        .setDeletions(util.Arrays.asList(new AlterUserScramCredentialsRequestData.ScramCredentialDeletion().setName(user1).setMechanism(ScramMechanism.SCRAM_SHA_256.`type`)))
+        .setUpsertions(util.Arrays.asList(new AlterUserScramCredentialsRequestData.ScramCredentialUpsertion().setName(user2).setMechanism(ScramMechanism.SCRAM_SHA_512.`type`)))).build()
+    val response = sendAlterUserScramCredentialsRequest(request)
+
+    val results = response.data.results
+    assertEquals(2, results.size)
+    checkAllErrorsAlteringCredentials(results, Errors.CLUSTER_AUTHORIZATION_FAILED, "when not authorized")
+  }
+
   @Test
   def testAlterSameThingTwice(): Unit = {
     val deletion1 = new AlterUserScramCredentialsRequestData.ScramCredentialDeletion().setName(user1).setMechanism(ScramMechanism.SCRAM_SHA_256.`type`)
@@ -320,17 +356,17 @@ class AlterUserScramCredentialsRequestTest extends BaseRequestTest {
   }
 
   private def checkAllErrorsAlteringCredentials(resultsToCheck: util.List[AlterUserScramCredentialsResult], expectedError: Errors, contextMsg: String) = {
-    assertEquals(0, resultsToCheck.asScala.filterNot(_.errorCode == expectedError.code).size,
-      s"Expected all '${expectedError.name}' errors when altering credentials $contextMsg")
+    assertEquals(s"Expected all '${expectedError.name}' errors when altering credentials $contextMsg",
+      0, resultsToCheck.asScala.filterNot(_.errorCode == expectedError.code).size)
   }
 
   private def checkNoErrorsAlteringCredentials(resultsToCheck: util.List[AlterUserScramCredentialsResult]) = {
-    assertEquals(0, resultsToCheck.asScala.filterNot(_.errorCode == Errors.NONE.code).size,
-      "Expected no error when altering credentials")
+    assertEquals("Expected no error when altering credentials",
+      0, resultsToCheck.asScala.filterNot(_.errorCode == Errors.NONE.code).size)
   }
 
   private def checkUserAppearsInAlterResults(resultsToCheck: util.List[AlterUserScramCredentialsResult], user: String) = {
-    assertTrue(resultsToCheck.asScala.exists(_.user == user), s"Expected result to contain '$user'")
+    assertTrue(s"Expected result to contain '$user'", resultsToCheck.asScala.exists(_.user == user))
   }
 
   private def describeAllWithNoTopLevelErrorConfirmed() = {
@@ -341,30 +377,30 @@ class AlterUserScramCredentialsRequestTest extends BaseRequestTest {
   }
 
   private def checkNoTopLevelErrorDescribingCredentials(responseToCheck: DescribeUserScramCredentialsResponse) = {
-    assertEquals(Errors.NONE.code, responseToCheck.data.errorCode, "Expected no top-level error when describing the credentials")
+    assertEquals("Expected no top-level error when describing the credentials", Errors.NONE.code, responseToCheck.data.errorCode)
   }
 
   private def checkUserHasTwoCredentials(resultsToCheck: util.List[DescribeUserScramCredentialsResult], user: String) = {
-    assertTrue(resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.size == 2 && result.errorCode == Errors.NONE.code),
-      s"Expected result to contain '$user' with 2 credentials: $resultsToCheck")
-    assertTrue(resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.asScala.exists(info =>
+    assertTrue(s"Expected result to contain '$user' with 2 credentials: $resultsToCheck",
+      resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.size == 2 && result.errorCode == Errors.NONE.code))
+    assertTrue(s"Expected result to contain '$user' with SCRAM_SHA_256/4096 and SCRAM_SHA_512/8192 credentials: $resultsToCheck",
+      resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.asScala.exists(info =>
         info.mechanism == ScramMechanism.SCRAM_SHA_256.`type` && info.iterations == 4096)
         && result.credentialInfos.asScala.exists(info =>
-        info.mechanism == ScramMechanism.SCRAM_SHA_512.`type` && info.iterations == 8192)),
-      s"Expected result to contain '$user' with SCRAM_SHA_256/4096 and SCRAM_SHA_512/8192 credentials: $resultsToCheck")
+        info.mechanism == ScramMechanism.SCRAM_SHA_512.`type` && info.iterations == 8192)))
   }
 
   private def checkForSingleSha512Iterations8192Credential(resultsToCheck: util.List[DescribeUserScramCredentialsResult], user: String) = {
-    assertTrue(resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.size == 1 && result.errorCode == Errors.NONE.code),
-      s"Expected result to contain '$user' with 1 credential: $resultsToCheck")
-    assertTrue(resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.asScala.exists(info =>
-        info.mechanism == ScramMechanism.SCRAM_SHA_512.`type` && info.iterations == 8192)),
-      s"Expected result to contain '$user' with SCRAM_SHA_512/8192 credential: $resultsToCheck")
+    assertTrue(s"Expected result to contain '$user' with 1 credential: $resultsToCheck",
+      resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.size == 1 && result.errorCode == Errors.NONE.code))
+    assertTrue(s"Expected result to contain '$user' with SCRAM_SHA_512/8192 credential: $resultsToCheck",
+      resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.asScala.exists(info =>
+        info.mechanism == ScramMechanism.SCRAM_SHA_512.`type` && info.iterations == 8192)))
   }
 
   private def checkDescribeForError(resultsToCheck: util.List[DescribeUserScramCredentialsResult], user: String, expectedError: Errors) = {
-    assertTrue(resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.size == 0 && result.errorCode == expectedError.code),
-      s"Expected result to contain '$user' with a ${expectedError.name} error: $resultsToCheck")
+    assertTrue(s"Expected result to contain '$user' with a ${expectedError.name} error: $resultsToCheck",
+      resultsToCheck.asScala.exists(result => result.user == user && result.credentialInfos.size == 0 && result.errorCode == expectedError.code))
   }
 }
 
