@@ -17,6 +17,8 @@
 
 package kafka.server
 
+import java.util.Properties
+
 import kafka.api.{ApiVersion, KAFKA_0_8_2}
 import kafka.cluster.EndPoint
 import kafka.log.LogConfig
@@ -27,18 +29,9 @@ import org.apache.kafka.common.metrics.Sensor
 import org.apache.kafka.common.network.ListenerName
 import org.apache.kafka.common.record.Records
 import org.apache.kafka.common.security.auth.SecurityProtocol
-import org.apache.kafka.raft.RaftConfig
-import org.apache.kafka.raft.RaftConfig.{AddressSpec, InetAddressSpec, UNKNOWN_ADDRESS_SPEC_INSTANCE}
-import org.junit.jupiter.api.Assertions._
-import org.junit.jupiter.api.Test
-import java.net.InetSocketAddress
-import java.util
-import java.util.{Collections, Properties}
-
-import org.apache.kafka.common.Node
-import org.junit.jupiter.api.function.Executable
-
-import scala.jdk.CollectionConverters._
+import org.junit.Assert._
+import org.junit.Test
+import org.scalatest.Assertions.intercept
 
 class KafkaConfigTest {
 
@@ -112,19 +105,21 @@ class KafkaConfigTest {
     val cfg1 = KafkaConfig.fromProps(props1)
     val cfg2 = KafkaConfig.fromProps(props2)
     val cfg3 = KafkaConfig.fromProps(props3)
-    assertEquals(-1, cfg1.logRetentionTimeMillis, "Should be -1")
-    assertEquals(-1, cfg2.logRetentionTimeMillis, "Should be -1")
-    assertEquals(-1, cfg3.logRetentionTimeMillis, "Should be -1")
+    assertEquals("Should be -1", -1, cfg1.logRetentionTimeMillis)
+    assertEquals("Should be -1", -1, cfg2.logRetentionTimeMillis)
+    assertEquals("Should be -1", -1, cfg3.logRetentionTimeMillis)
 
     props4.put("log.retention.ms", "-1")
     props4.put("log.retention.minutes", "30")
 
     val cfg4 = KafkaConfig.fromProps(props4)
-    assertEquals(-1, cfg4.logRetentionTimeMillis, "Should be -1")
+    assertEquals("Should be -1", -1, cfg4.logRetentionTimeMillis)
 
     props5.put("log.retention.ms", "0")
 
-    assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props5))
+    intercept[IllegalArgumentException] {
+      KafkaConfig.fromProps(props5)
+    }
   }
 
   @Test
@@ -137,9 +132,15 @@ class KafkaConfigTest {
     props2.put("log.retention.minutes", "0")
     props3.put("log.retention.hours", "0")
 
-    assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props1))
-    assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props2))
-    assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props3))
+    intercept[IllegalArgumentException] {
+      KafkaConfig.fromProps(props1)
+    }
+    intercept[IllegalArgumentException] {
+      KafkaConfig.fromProps(props2)
+    }
+    intercept[IllegalArgumentException] {
+      KafkaConfig.fromProps(props3)
+    }
 
   }
 
@@ -218,12 +219,12 @@ class KafkaConfigTest {
 
     // listeners with duplicate port
     props.put(KafkaConfig.ListenersProp, "PLAINTEXT://localhost:9091,SSL://localhost:9091")
-    var caught = assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props))
+    var caught = intercept[IllegalArgumentException] { KafkaConfig.fromProps(props) }
     assertTrue(caught.getMessage.contains("Each listener must have a different port"))
 
     // listeners with duplicate name
     props.put(KafkaConfig.ListenersProp, "PLAINTEXT://localhost:9091,PLAINTEXT://localhost:9092")
-    caught = assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props))
+    caught = intercept[IllegalArgumentException] { KafkaConfig.fromProps(props) }
     assertTrue(caught.getMessage.contains("Each listener must have a different name"))
 
     // advertised listeners can have duplicate ports
@@ -235,7 +236,7 @@ class KafkaConfigTest {
 
     // but not duplicate names
     props.put(KafkaConfig.AdvertisedListenersProp, "HOST://localhost:9091,HOST://localhost:9091")
-    caught = assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props))
+    caught = intercept[IllegalArgumentException] { KafkaConfig.fromProps(props) }
     assertTrue(caught.getMessage.contains("Each listener must have a different name"))
   }
 
@@ -264,30 +265,6 @@ class KafkaConfigTest {
     assertFalse(interBrokerListenerName.value().equals(controlEndpoint.listenerName.value()))
   }
 
-  @Test
-  def testControllerListenerName() = {
-    val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect)
-    props.put(KafkaConfig.ListenersProp, "PLAINTEXT://localhost:0,CONTROLPLANE://localhost:4000,CONTROLLER://localhost:5000")
-    props.put(KafkaConfig.ListenerSecurityProtocolMapProp, "PLAINTEXT:PLAINTEXT,CONTROLPLANE:SSL,CONTROLLER:SASL_SSL")
-    props.put(KafkaConfig.AdvertisedListenersProp, "PLAINTEXT://localhost:0,CONTROLPLANE://localhost:4000")
-    props.put(KafkaConfig.ControlPlaneListenerNameProp, "CONTROLPLANE")
-    props.put(KafkaConfig.ControllerListenerNamesProp, "CONTROLLER")
-    assertTrue(isValidKafkaConfig(props))
-
-    val serverConfig = KafkaConfig.fromProps(props)
-    val controlPlaneEndpoint = serverConfig.controlPlaneListener.get
-    assertEquals("localhost", controlPlaneEndpoint.host)
-    assertEquals(4000, controlPlaneEndpoint.port)
-    assertEquals(SecurityProtocol.SSL, controlPlaneEndpoint.securityProtocol)
-
-    val controllerEndpoints = serverConfig.controllerListeners
-    assertEquals(1, controllerEndpoints.size)
-    val controllerEndpoint = controllerEndpoints.iterator.next()
-    assertEquals("localhost", controllerEndpoint.host)
-    assertEquals(5000, controllerEndpoint.port)
-    assertEquals(SecurityProtocol.SASL_SSL, controllerEndpoint.securityProtocol)
-  }
-
   @Test
   def testBadListenerProtocol(): Unit = {
     val props = new Properties()
@@ -421,7 +398,7 @@ class KafkaConfigTest {
     val conf2 = KafkaConfig.fromProps(props)
     assertEquals(listenerListToEndPoints("PLAINTEXT://:1111"), conf2.listeners)
     assertEquals(listenerListToEndPoints("PLAINTEXT://:1111"), conf2.advertisedListeners)
-    assertNull(conf2.listeners.find(_.securityProtocol == SecurityProtocol.PLAINTEXT).get.host)
+    assertEquals(null, conf2.listeners.find(_.securityProtocol == SecurityProtocol.PLAINTEXT).get.host)
 
     // configuration with advertised host and port, and no advertised listeners
     props.put(KafkaConfig.AdvertisedHostNameProp, "otherhost")
@@ -496,7 +473,9 @@ class KafkaConfigTest {
     val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect, port = 8181)
     props.put(KafkaConfig.UncleanLeaderElectionEnableProp, "invalid")
 
-    assertThrows(classOf[ConfigException], () => KafkaConfig.fromProps(props))
+    intercept[ConfigException] {
+      KafkaConfig.fromProps(props)
+    }
   }
 
   @Test
@@ -547,7 +526,9 @@ class KafkaConfigTest {
   def testInvalidCompressionType(): Unit = {
     val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect, port = 8181)
     props.put(KafkaConfig.CompressionTypeProp, "abc")
-    assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props))
+    intercept[IllegalArgumentException] {
+      KafkaConfig.fromProps(props)
+    }
   }
 
   @Test
@@ -555,7 +536,9 @@ class KafkaConfigTest {
     val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect, port = 8181)
     props.put(KafkaConfig.ListenersProp, "SSL://localhost:0")
     props.put(KafkaConfig.InterBrokerSecurityProtocolProp, SecurityProtocol.PLAINTEXT.toString)
-    assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props))
+    intercept[IllegalArgumentException] {
+      KafkaConfig.fromProps(props)
+    }
   }
 
   @Test
@@ -571,11 +554,11 @@ class KafkaConfigTest {
     val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect, port = 8181)
     props.put(KafkaConfig.ListenersProp, "TRACE://localhost:9091,SSL://localhost:9093")
     props.put(KafkaConfig.AdvertisedListenersProp, "PLAINTEXT://localhost:9092")
-    var caught = assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props))
+    var caught = intercept[IllegalArgumentException] { KafkaConfig.fromProps(props) }
     assertTrue(caught.getMessage.contains("No security protocol defined for listener TRACE"))
 
     props.put(KafkaConfig.ListenerSecurityProtocolMapProp, "PLAINTEXT:PLAINTEXT,TRACE:PLAINTEXT,SSL:SSL")
-    caught = assertThrows(classOf[IllegalArgumentException], () => KafkaConfig.fromProps(props))
+    caught = intercept[IllegalArgumentException] { KafkaConfig.fromProps(props) }
     assertTrue(caught.getMessage.contains("advertised.listeners listener names must be equal to or a subset of the ones defined in listeners"))
   }
 
@@ -595,7 +578,9 @@ class KafkaConfigTest {
           assertEquals(messageFormatVersion, config.logMessageFormatVersion)
           assertEquals(interBrokerVersion, config.interBrokerProtocolVersion)
         } else {
-          assertThrows(classOf[IllegalArgumentException], () => buildConfig(interBrokerVersion, messageFormatVersion))
+          intercept[IllegalArgumentException] {
+            buildConfig(interBrokerVersion, messageFormatVersion)
+          }
         }
       }
     }
@@ -645,15 +630,6 @@ class KafkaConfigTest {
         case KafkaConfig.ConnectionSetupTimeoutMsProp => assertPropertyInvalid(baseProperties, name, "not_a_number")
         case KafkaConfig.ConnectionSetupTimeoutMaxMsProp => assertPropertyInvalid(baseProperties, name, "not_a_number")
 
-        // KRaft mode configs
-        case KafkaConfig.ProcessRolesProp => // ignore
-        case KafkaConfig.InitialBrokerRegistrationTimeoutMsProp => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case KafkaConfig.BrokerHeartbeatIntervalMsProp => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case KafkaConfig.BrokerSessionTimeoutMsProp => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case KafkaConfig.NodeIdProp => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case KafkaConfig.MetadataLogDirProp => // ignore string
-        case KafkaConfig.ControllerListenerNamesProp => // ignore string
-
         case KafkaConfig.AuthorizerClassNameProp => //ignore string
         case KafkaConfig.CreateTopicPolicyClassNameProp => //ignore string
 
@@ -779,7 +755,6 @@ class KafkaConfigTest {
         case KafkaConfig.SslPrincipalMappingRulesProp => // ignore string
 
         //Sasl Configs
-        case KafkaConfig.SaslMechanismControllerProtocolProp => // ignore
         case KafkaConfig.SaslMechanismInterBrokerProtocolProp => // ignore
         case KafkaConfig.SaslEnabledMechanismsProp =>
         case KafkaConfig.SaslClientCallbackHandlerClassProp =>
@@ -810,8 +785,7 @@ class KafkaConfigTest {
         case KafkaConfig.PasswordEncoderIterationsProp => assertPropertyInvalid(baseProperties, name, "not_a_number", "-1", "0")
 
         //delegation token configs
-        case KafkaConfig.DelegationTokenSecretKeyAliasProp => // ignore
-        case KafkaConfig.DelegationTokenSecretKeyProp => // ignore
+        case KafkaConfig.DelegationTokenMasterKeyProp => // ignore
         case KafkaConfig.DelegationTokenMaxLifeTimeProp => assertPropertyInvalid(baseProperties, name, "not_a_number", "0")
         case KafkaConfig.DelegationTokenExpiryTimeMsProp => assertPropertyInvalid(baseProperties, name, "not_a_number", "0")
         case KafkaConfig.DelegationTokenExpiryCheckIntervalMsProp => assertPropertyInvalid(baseProperties, name, "not_a_number", "0")
@@ -820,17 +794,6 @@ class KafkaConfigTest {
         case KafkaConfig.KafkaMetricsReporterClassesProp => // ignore
         case KafkaConfig.KafkaMetricsPollingIntervalSecondsProp => //ignore
 
-        case KafkaConfig.SaslServerMaxReceiveSizeProp => assertPropertyInvalid(baseProperties, name, "not_a_number")
-
-        // Raft Quorum Configs
-        case RaftConfig.QUORUM_VOTERS_CONFIG => // ignore string
-        case RaftConfig.QUORUM_ELECTION_TIMEOUT_MS_CONFIG => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case RaftConfig.QUORUM_FETCH_TIMEOUT_MS_CONFIG => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case RaftConfig.QUORUM_ELECTION_BACKOFF_MAX_MS_CONFIG => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case RaftConfig.QUORUM_LINGER_MS_CONFIG => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case RaftConfig.QUORUM_REQUEST_TIMEOUT_MS_CONFIG => assertPropertyInvalid(baseProperties, name, "not_a_number")
-        case RaftConfig.QUORUM_RETRY_BACKOFF_MS_CONFIG => assertPropertyInvalid(baseProperties, name, "not_a_number")
-
         case _ => assertPropertyInvalid(baseProperties, name, "not_a_number", "-1")
       }
     }
@@ -962,7 +925,7 @@ class KafkaConfigTest {
     assertEquals(24 * 60L * 60L * 1000L, config.delegationTokenExpiryTimeMs)
     assertEquals(1 * 60L * 1000L * 60, config.delegationTokenExpiryCheckIntervalMs)
 
-    defaults.put(KafkaConfig.DelegationTokenSecretKeyProp, "1234567890")
+    defaults.put(KafkaConfig.DelegationTokenMasterKeyProp, "1234567890")
     val config1 = KafkaConfig.fromProps(defaults)
     assertEquals(true, config1.tokenAuthEnabled)
   }
@@ -987,260 +950,13 @@ class KafkaConfigTest {
   }
 
   private def assertPropertyInvalid(validRequiredProps: => Properties, name: String, values: Any*): Unit = {
-    values.foreach { value =>
+    values.foreach((value) => {
       val props = validRequiredProps
       props.setProperty(name, value.toString)
-
-      val buildConfig: Executable = () => KafkaConfig.fromProps(props)
-      assertThrows(classOf[Exception], buildConfig,
-      s"Expected exception for property `$name` with invalid value `$value` was not thrown")
-    }
-  }
-
-  def assertDistinctControllerAndAdvertisedListeners(): Unit = {
-    val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.AdvertisedListenersProp, "PLAINTEXT://A:9092,SSL://B:9093")
-    // Valid now
-    assertTrue(isValidKafkaConfig(props))
-
-    // Still valid
-    val controllerListeners = "SASL_SSL"
-    props.put(KafkaConfig.ControllerListenerNamesProp, controllerListeners)
-    assertTrue(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def assertAllControllerListenerCannotBeAdvertised(): Unit = {
-    val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.AdvertisedListenersProp, listeners)
-    // Valid now
-    assertTrue(isValidKafkaConfig(props))
-
-    // Invalid now
-    props.put(KafkaConfig.ControllerListenerNamesProp, "PLAINTEXT,SSL,SASL_SSL")
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def assertEvenOneControllerListenerCannotBeAdvertised(): Unit = {
-    val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.AdvertisedListenersProp, listeners)
-    // Valid now
-    assertTrue(isValidKafkaConfig(props))
-
-    // Invalid now
-    props.put(KafkaConfig.ControllerListenerNamesProp, "PLAINTEXT")
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testControllerQuorumVoterStringsToNodes(): Unit = {
-    assertThrows(classOf[ConfigException], () => RaftConfig.quorumVoterStringsToNodes(Collections.singletonList("")))
-    assertEquals(Seq(new Node(3000, "example.com", 9093)),
-      RaftConfig.quorumVoterStringsToNodes(util.Arrays.asList("3000@example.com:9093")).asScala.toSeq)
-    assertEquals(Seq(new Node(3000, "example.com", 9093),
-      new Node(3001, "example.com", 9094)),
-      RaftConfig.quorumVoterStringsToNodes(util.Arrays.asList("3000@example.com:9093","3001@example.com:9094")).asScala.toSeq)
-  }
-
-  @Test
-  def testInvalidQuorumVoterConfig(): Unit = {
-    assertInvalidQuorumVoters("1")
-    assertInvalidQuorumVoters("1@")
-    assertInvalidQuorumVoters("1:")
-    assertInvalidQuorumVoters("blah@")
-    assertInvalidQuorumVoters("1@kafka1")
-    assertInvalidQuorumVoters("1@kafka1:9092,")
-    assertInvalidQuorumVoters("1@kafka1:9092,")
-    assertInvalidQuorumVoters("1@kafka1:9092,2")
-    assertInvalidQuorumVoters("1@kafka1:9092,2@")
-    assertInvalidQuorumVoters("1@kafka1:9092,2@blah")
-    assertInvalidQuorumVoters("1@kafka1:9092,2@blah,")
-    assertInvalidQuorumVoters("1@kafka1:9092:1@kafka2:9092")
-  }
-
-  private def assertInvalidQuorumVoters(value: String): Unit = {
-    val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect)
-    props.put(RaftConfig.QUORUM_VOTERS_CONFIG, value)
-    assertThrows(classOf[ConfigException], () => KafkaConfig.fromProps(props))
-  }
-
-  @Test
-  def testValidQuorumVotersConfig(): Unit = {
-    val expected = new util.HashMap[Integer, AddressSpec]()
-    assertValidQuorumVoters("", expected)
-
-    expected.put(1, new InetAddressSpec(new InetSocketAddress("127.0.0.1", 9092)))
-    assertValidQuorumVoters("1@127.0.0.1:9092", expected)
-
-    expected.clear()
-    expected.put(1, UNKNOWN_ADDRESS_SPEC_INSTANCE)
-    assertValidQuorumVoters("1@0.0.0.0:0", expected)
-
-    expected.clear()
-    expected.put(1, new InetAddressSpec(new InetSocketAddress("kafka1", 9092)))
-    expected.put(2, new InetAddressSpec(new InetSocketAddress("kafka2", 9092)))
-    expected.put(3, new InetAddressSpec(new InetSocketAddress("kafka3", 9092)))
-    assertValidQuorumVoters("1@kafka1:9092,2@kafka2:9092,3@kafka3:9092", expected)
-  }
-
-  private def assertValidQuorumVoters(value: String, expectedVoters: util.Map[Integer, AddressSpec]): Unit = {
-    val props = TestUtils.createBrokerConfig(0, TestUtils.MockZkConnect)
-    props.put(RaftConfig.QUORUM_VOTERS_CONFIG, value)
-    val raftConfig = new RaftConfig(KafkaConfig.fromProps(props))
-    assertEquals(expectedVoters, raftConfig.quorumVoterConnections())
-  }
-
-  @Test
-  def testAcceptsLargeNodeIdForRaftBasedCase(): Unit = {
-    // Generation of Broker IDs is not supported when using Raft-based controller quorums,
-    // so pick a broker ID greater than reserved.broker.max.id, which defaults to 1000,
-    // and make sure it is allowed despite broker.id.generation.enable=true (true is the default)
-    val largeBrokerId = 2000
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "broker")
-    props.put(KafkaConfig.NodeIdProp, largeBrokerId.toString)
-    assertTrue(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testRejectsNegativeNodeIdForRaftBasedBrokerCaseWithAutoGenEnabled(): Unit = {
-    // -1 is the default for both node.id and broker.id
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "broker")
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testRejectsNegativeNodeIdForRaftBasedControllerCaseWithAutoGenEnabled(): Unit = {
-    // -1 is the default for both node.id and broker.id
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "controller")
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testRejectsNegativeNodeIdForRaftBasedCaseWithAutoGenDisabled(): Unit = {
-    // -1 is the default for both node.id and broker.id
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "broker")
-    props.put(KafkaConfig.BrokerIdGenerationEnableProp, "false")
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testRejectsLargeNodeIdForZkBasedCaseWithAutoGenEnabled(): Unit = {
-    // Generation of Broker IDs is supported when using ZooKeeper-based controllers,
-    // so pick a broker ID greater than reserved.broker.max.id, which defaults to 1000,
-    // and make sure it is not allowed with broker.id.generation.enable=true (true is the default)
-    val largeBrokerId = 2000
-    val props = TestUtils.createBrokerConfig(largeBrokerId, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.AdvertisedListenersProp, listeners)
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testAcceptsNegativeOneNodeIdForZkBasedCaseWithAutoGenEnabled(): Unit = {
-    // -1 is the default for both node.id and broker.id; it implies "auto-generate" and should succeed
-    val props = TestUtils.createBrokerConfig(-1, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.AdvertisedListenersProp, listeners)
-    assertTrue(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testRejectsNegativeTwoNodeIdForZkBasedCaseWithAutoGenEnabled(): Unit = {
-    // -1 implies "auto-generate" and should succeed, but -2 does not and should fail
-    val negativeTwoNodeId = -2
-    val props = TestUtils.createBrokerConfig(negativeTwoNodeId, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.AdvertisedListenersProp, listeners)
-    props.put(KafkaConfig.NodeIdProp, negativeTwoNodeId.toString)
-    props.put(KafkaConfig.BrokerIdProp, negativeTwoNodeId.toString)
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testAcceptsLargeNodeIdForZkBasedCaseWithAutoGenDisabled(): Unit = {
-    // Ensure a broker ID greater than reserved.broker.max.id, which defaults to 1000,
-    // is allowed with broker.id.generation.enable=false
-    val largeBrokerId = 2000
-    val props = TestUtils.createBrokerConfig(largeBrokerId, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.AdvertisedListenersProp, listeners)
-    props.put(KafkaConfig.BrokerIdGenerationEnableProp, "false")
-    assertTrue(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testRejectsNegativeNodeIdForZkBasedCaseWithAutoGenDisabled(): Unit = {
-    // -1 is the default for both node.id and broker.id
-    val props = TestUtils.createBrokerConfig(-1, TestUtils.MockZkConnect, port = TestUtils.MockZkPort)
-    val listeners = "PLAINTEXT://A:9092,SSL://B:9093,SASL_SSL://C:9094"
-    props.put(KafkaConfig.ListenersProp, listeners)
-    props.put(KafkaConfig.BrokerIdGenerationEnableProp, "false")
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testZookeeperConnectRequiredIfEmptyProcessRoles(): Unit = {
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "")
-    props.put(KafkaConfig.ListenersProp, "PLAINTEXT://127.0.0.1:9092")
-    assertFalse(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testZookeeperConnectNotRequiredIfNonEmptyProcessRoles(): Unit = {
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "broker")
-    props.put(KafkaConfig.ListenersProp, "PLAINTEXT://127.0.0.1:9092")
-    props.put(KafkaConfig.NodeIdProp, "1")
-    assertTrue(isValidKafkaConfig(props))
-  }
-
-  @Test
-  def testCustomMetadataLogDir(): Unit = {
-    val metadataDir = "/path/to/metadata/dir"
-    val dataDir = "/path/to/data/dir"
-
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "broker")
-    props.put(KafkaConfig.MetadataLogDirProp, metadataDir)
-    props.put(KafkaConfig.LogDirProp, dataDir)
-    props.put(KafkaConfig.NodeIdProp, "1")
-    assertTrue(isValidKafkaConfig(props))
-
-    val config = KafkaConfig.fromProps(props)
-    assertEquals(metadataDir, config.metadataLogDir)
-    assertEquals(Seq(dataDir), config.logDirs)
-  }
-
-  @Test
-  def testDefaultMetadataLogDir(): Unit = {
-    val dataDir1 = "/path/to/data/dir/1"
-    val dataDir2 = "/path/to/data/dir/2"
-
-    val props = new Properties()
-    props.put(KafkaConfig.ProcessRolesProp, "broker")
-    props.put(KafkaConfig.LogDirProp, s"$dataDir1,$dataDir2")
-    props.put(KafkaConfig.NodeIdProp, "1")
-    assertTrue(isValidKafkaConfig(props))
-
-    val config = KafkaConfig.fromProps(props)
-    assertEquals(dataDir1, config.metadataLogDir)
-    assertEquals(Seq(dataDir1, dataDir2), config.logDirs)
+      intercept[Exception] {
+        KafkaConfig.fromProps(props)
+      }
+    })
   }
 
 }
